@@ -21,12 +21,13 @@ module:hook("iq-get/host/urn:xmpp:ping:ping", ping_handler);
 
 local datetime = require "util.datetime".datetime;
 
-function ping_command_handler (self, data, state)
+function ping_command_handler (self, data, state) -- luacheck: ignore 212
 	local now = datetime();
 	return { info = "Pong\n"..now, status = "completed" };
 end
 
+module:depends "adhoc";
 local adhoc_new = module:require "adhoc".new;
 local descriptor = adhoc_new("Ping", "ping", ping_command_handler);
-module:add_item ("adhoc", descriptor);
+module:provides("adhoc", descriptor);
 
