@@ -143,7 +143,7 @@ end
 
 -- actor is the attribute table
 local function add_item(x, affiliation, role, jid, nick, actor_nick, actor_jid, reason)
-	x:tag("item", {affiliation = affiliation; role = role; jid = jid; nick = nick;})
+	x:tag("item", {affiliation = affiliation or "none"; role = role; jid = jid; nick = nick;})
 	if actor_nick or actor_jid then
 		x:tag("actor", {nick = actor_nick; jid = actor_jid;}):up()
 	end
@@ -336,12 +336,12 @@ function room_mt:send_occupant_list(to, filter)
 end
 
 function room_mt:get_disco_info(stanza)
-	local node = stanza.tags[1].attr.node or "";
+	local node = stanza.tags[1].attr.node;
 	local reply = st.reply(stanza):tag("query", { xmlns = "http://jabber.org/protocol/disco#info", node = node });
 	local event_name = "muc-disco#info";
 	local event_data = { room = self, reply = reply, stanza = stanza };
 
-	if node ~= "" then
+	if node and node ~= "" then
 		event_name = event_name.."/"..node;
 	else
 		event_data.form = dataform.new {
@@ -976,7 +976,7 @@ function room_mt:handle_admin_query_get_command(origin, stanza)
 		-- e.g. an admin can't ask for a list of owners
 		local affiliation_rank = valid_affiliations[affiliation or "none"];
 		if (affiliation_rank >= valid_affiliations.admin and affiliation_rank >= _aff_rank)
-		or (self:get_whois() == "anyone") then
+		or (self:get_members_only() and self:get_whois() == "anyone" and affiliation_rank >= valid_affiliations.member) then
 			local reply = st.reply(stanza):query("http://jabber.org/protocol/muc#admin");
 			for jid in self:each_affiliation(_aff or "none") do
 				local nick = self:get_registered_nick(jid);
@@ -1326,8 +1326,8 @@ function room_mt:set_affiliation(actor, jid, affiliation, reason, data)
 			if occupant.role == nil then
 				module:fire_event("muc-occupant-left", {room = self; nick = occupant.nick; occupant = occupant;});
 			elseif is_semi_anonymous and
-				(old_role == "moderator" and occupant.role ~= "moderator") or
-				(old_role ~= "moderator" and occupant.role == "moderator") then -- Has gained or lost moderator status
+				((old_role == "moderator" and occupant.role ~= "moderator") or
+				(old_role ~= "moderator" and occupant.role == "moderator")) then -- Has gained or lost moderator status
 				-- Send everyone else's presences (as jid visibility has changed)
 				for real_jid in occupant:each_session() do
 					self:send_occupant_list(real_jid, function(occupant_jid, occupant) --luacheck: ignore 212 433
@@ -1449,6 +1449,7 @@ function room_mt:freeze(live)
 		frozen = {
 			_jid = self.jid;
 			_data = self._data;
+			_affiliation_data = self._affiliation_data;
 		};
 		for user, affiliation in pairs(self._affiliations) do
 			frozen[user] = affiliation;
