@@ -13,11 +13,11 @@ local next = next;
 local pairs = pairs;
 local json = require "util.json";
 local os_getenv = os.getenv;
+local error = error;
 
 prosody = {};
 local dm = require "util.datamanager"
 
-module "prosody_files"
 
 local function is_dir(path) return lfs.attributes(path, "mode") == "directory"; end
 local function is_file(path) return lfs.attributes(path, "mode") == "file"; end
@@ -87,7 +87,7 @@ local function decode_user(item)
 	return userdata;
 end
 
-function reader(input)
+local function reader(input)
 	local path = clean_path(assert(input.path, "no input.path specified"));
 	assert(is_dir(path), "input.path is not a directory");
 	local iter = coroutine.wrap(function()handle_root_dir(path);end);
@@ -95,15 +95,18 @@ function reader(input)
 	local iter = mtools.sorted {
 		reader = function()
 			local x = iter();
-			if x then
+			while x do
 				dm.set_data_path(path);
 				local err;
 				x.data, err = dm.load(x.user, x.host, x.store);
 				if x.data == nil and err then
-					error(("Error loading data at path %s for %s@%s (%s store)")
-						:format(path, x.user or "<nil>", x.host or "<nil>", x.store or "<nil>"), 0);
+					local p = dm.getpath(x.user, x.host, x.store);
+					print(("Error loading data at path %s for %s@%s (%s store): %s")
+						:format(p, x.user or "<nil>", x.host or "<nil>", x.store or "<nil>", err or "<nil>"));
+				else
+					return x;
 				end
-				return x;
+				x = iter();
 			end
 		end;
 		sorter = function(a, b)
@@ -123,7 +126,7 @@ function reader(input)
 	end
 end
 
-function writer(output)
+local function writer(output)
 	local path = clean_path(assert(output.path, "no output.path specified"));
 	assert(is_dir(path), "output.path is not a directory");
 	return function(item)
@@ -135,4 +138,7 @@ function writer(output)
 	end
 end
 
-return _M;
+return {
+	reader = reader;
+	writer = writer;
+}
