@@ -11,11 +11,16 @@ if ok then return crand; end
 
 local urandom, urandom_err = io.open("/dev/urandom", "r");
 
-local function seed()
-end
-
 local function bytes(n)
-	return urandom:read(n);
+	local data, err = urandom:read(n);
+	if not data then
+		if err then
+			error("Unable to retrieve data from secure random number generator (/dev/urandom): "..tostring(err));
+		else
+			error("Secure random number generator (/dev/urandom) returned an end-of-file condition");
+		end
+	end
+	return data;
 end
 
 if not urandom then
@@ -25,7 +30,6 @@ if not urandom then
 end
 
 return {
-	seed = seed;
 	bytes = bytes;
 	_source = "/dev/urandom";
 };
